

import os
import sys
import subprocess

import site
from pathlib import Path

import torch
from torch.utils.data.dataloader import DataLoader
from torch.utils.data.dataset import Subset

#site.addsitedir(Path(__file__).parent.parent.absolute().as_posix())
#from predtuner import PipedBinaryApp, config_pylogger



class Benchmark:

    def __init__(self, binary_path, test_accuracy):

        self.binary_path = binary_path
        self.test_accuracy = test_accuracy
        self.epsilon = 0.05 # Adding some slack for accuracy difference


    def getPath(self):
        return self.binary_path

    
    def readAccuracy(self, accuracy_file):

        f = open(accuracy_file, "r") # File with final benchmark accuracy 
        acc_str = f.read()
        return float(acc_str)
    
        
    def runKeras(self):

        # Test Bechmark accuracy with pretrained weights (hpvm_relaod)
        run_cmd = "python3 " + self.binary_path + " keras_reload "
        try:
            subprocess.call(run_cmd, shell=True)
        except:
            return False

        accuracy = self.readAccuracy("final_accuracy")

        print ("accuracy = ", accuracy, " test_accuracy = ", self.test_accuracy) 

        test_success = False
        if (abs(self.test_accuracy - accuracy) < self.epsilon):
            print ("Test for " + self. binary_path + " Passed ")
            test_success = True
        else:
            print ("Test Failed for " + self.binary_path)
            test_success = False

        return test_success


    def runHPVM(self):

        # Test Bechmark accuracy with pretrained weights (hpvm_relaod)
        run_cmd = "python3 " + self.binary_path + " keras_reload frontend compile compile_tuner"
        try:
            subprocess.call(run_cmd, shell=True)
        except:
            return False

        working_dir = open("working_dir.txt").read()
        cur_dir = os.getcwd()
        
        os.chdir(working_dir)
        binary_path =  "./HPVM_binary"
        
        try:
            subprocess.call(binary_path, shell=True)
        except:
            return False
        
        accuracy = self.readAccuracy("final_accuracy")
        print ("accuracy = ", accuracy, " test_accuracy = ", self.test_accuracy) 

        test_success = False
        if (abs(self.test_accuracy - accuracy) < self.epsilon):
            print ("Test for " + self. binary_path + " Passed ")
            test_success = True
        else:
            print ("Test Failed for " + self.binary_path)
            test_success = False

        os.chdir(cur_dir)  # Change back to original working directory
        
        return test_success


"""    
    def runApproxTuner(self):

        working_dir = open("working_dir.txt").read()
        cur_dir = os.getcwd()
        
        os.chdir(working_dir)
        binary_path =  "./HPVM_tuner_binary"

        full_binary_path = str(cur_dir) + "/" +  working_dir + "/" + binary_path
        full_json_path = str(cur_dir) + "/" + working_dir + "/tuner.json"
    
        app = PipedBinaryApp("TestHPVMApp", full_binary_path, full_json_path)
        # Tuning procedure is exactly the same as that for PyTorch DNN.
        # Please refer to `./tune_vgg16_cifar10.py` for details.
        tuner = app.get_tuner()
        tuner.tune(5000, 3.0, 3.0, True, 50, cost_model="cost_linear", qos_model="qos_p1")

        tuner.dump_configs("configs.json")
        fig = tuner.plot_configs(show_qos_loss=True)
        fig.savefig("configs.png", dpi=300)
        app.dump_hpvm_configs(tuner.best_configs, "hpvm_confs.txt")

        os.chdir(cur_dir)  # Change back to original working directory
"""
            
        

class BenchmarkTests:

    def __init__(self):

        self.benchmarks = []
        self.passed_tests = []
        self.failed_tests = []
        self.passed_hpvm_tests = []
        self.failed_hpvm_tests = []


    def addBenchmark(self, benchmark):

        self.benchmarks.append(benchmark)


    def runKerasTests(self):

        for benchmark in self.benchmarks:
            test_success = benchmark.runKeras()

            if not test_success:
                self.failed_tests.append(benchmark.getPath())
            else:
                self.passed_tests.append(benchmark.getPath())


    def runHPVMTests(self):

        for benchmark in self.benchmarks:
            test_success = benchmark.runHPVM()

            if not test_success:
                self.failed_hpvm_tests.append(benchmark.getPath())
            else:
                self.passed_hpvm_tests.append(benchmark.getPath())


    def printKerasSummary(self):

        failed_test_count = len(self.failed_tests)
        passed_test_count = len(self.passed_tests)
        
        print (" Tests Passed  = " + str(passed_test_count) + " / " + str(len(self.benchmarks)))
        print ("******* Passed Tests ** \n")
        for passed_test in self.passed_tests:
            print ("Passed: " + passed_test)

        print (" Tests Failed  = " + str(failed_test_count) + " / " + str(len(self.benchmarks)))
        print ("****** Failed Tests *** \n")
        for failed_test in self.failed_tests:
            print ("Failed: " + failed_test)
            

    def printHPVMSummary(self):

        failed_test_count = len(self.failed_hpvm_tests)
        passed_test_count = len(self.passed_hpvm_tests)
        
        print (" Tests Passed  = " + str(passed_test_count) + " / " + str(len(self.benchmarks)))
        print ("******* Passed Tests ** \n")
        for passed_test in self.passed_hpvm_tests:
            print ("Passed: " + passed_test)

        print (" Tests Failed  = " + str(failed_test_count) + " / " + str(len(self.benchmarks)))
        print ("****** Failed Tests *** \n")
        for failed_test in self.failed_hpvm_tests:
            print ("Failed: " + failed_test)
            

        
            
if __name__ == "__main__":

    if len(sys.argv) < 2:
        print ("Usage: python3 test_dnnbenchmarks.py ${work_dir}")

    work_dir = sys.argv[1]
    if not os.path.exists(work_dir):
        os.mkdir(work_dir)
    os.chdir(work_dir)
    
    testMgr = BenchmarkTests()
    AlexNet = Benchmark("../alexnet.py", 79.28)
    AlexNet_ImageNet = Benchmark("../alexnet_imagenet.py", 56.30)
    AlexNet2 = Benchmark("../alexnet2.py", 84.98)
    LeNet = Benchmark("../lenet.py", 98.70)
    MobileNet = Benchmark("../mobilenet_cifar10.py", 84.42)
    ResNet18 = Benchmark("../resnet18_cifar10.py", 89.56)
    ResNet50 = Benchmark("../resnet50_imagenet.py", 75.10)
    VGG16_cifar10 = Benchmark("../vgg16_cifar10.py", 89.96)
    VGG16_cifar100 = Benchmark("../vgg16_cifar100.py", 66.50)
    VGG16_ImageNet = Benchmark("../vgg16_imagenet.py", 69.46)

    testMgr.addBenchmark(AlexNet)
    #testMgr.addBenchmark(AlexNet_ImageNet)
    testMgr.addBenchmark(AlexNet2)
    testMgr.addBenchmark(LeNet)
    testMgr.addBenchmark(MobileNet)
    testMgr.addBenchmark(ResNet18)
    #testMgr.addBenchmark(ResNet50)
    testMgr.addBenchmark(VGG16_cifar10)
    testMgr.addBenchmark(VGG16_cifar100)
    #testMgr.addBenchmark(VGG16_ImageNet)

    #testMgr.runKerasTests()
    #testMgr.printKerasSummary()
    
    testMgr.runHPVMTests()
    testMgr.printHPVMSummary()

    
